/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Name.java,v 1.0 2003/08/04 08:07:19 diasz Exp $
 *
 * Date         Author  Changes
 */

/* Grammar for request travelling scenarios
a)
cpr_seq -> ENTER_SEND_REQUEST, EXIT_SEND_REQUEST, ENTER_RECEIVE_REQUEST, rest_cpr
rest_cpr -> SEND_FORWARD_EXC, RECEIVE OTHER|SEND_WR_PRIM_EXC, RECEIVE_EXCEPTION|SEND_REQ_EXP_EXC, RECEIVE_EXCEPTION|YES_REQUEST_RIGHT_PLACE,rest_rest_cpr
rest_rest_cpr -> EXIT_REPLY_ALREADY_THERE|START_WAIT_GET_STATE, STOP_WAIT_GET_STATE, EXIT_RECEIVE_REQUEST,rest_reply_cpr
rest_reply_cpr ->ENTER_SEND_REPLY, EXIT_SEND_REPLY, EXIT_RECEIVE_REPLY

b)
wpr_seq->cpr_seq

c)

sl_req -> ENTER_SEND_REQUEST, EXIT_SEND_REQUEST, ENTER_RECEIVE_REQUEST, rest_sl
rest_sl -> SEND_FORWARD_EXC|SEND_WR_PRIM_EXC|SEND_REQ_EXP_EXC|YES_REQUEST_RIGHT_PLACE,rest_rest_sl
rest_rest_sl -> EXIT_REPLY_ALREADY_THERE|EXIT_RECEIVE_REQUEST,rest_reply_sl
rest_reply_sl -> rest_reply_cpr

d)
ar_seq -> start, (SEND_REQ_EXP_EXC|normal_ar_seq)
start -> ENTER_SEND_REQUEST, EXIT_SEND_REQUEST
normal_ar_seq -> ENTER_RECEIVE_REQUEST_G, rest_ar
rest_ar -> GW_SEND_TRANSIENT| rest_rest_ar
rest_rest_ar -> rest_seq1, rest_seq2
rest_seq1 -> BEG_SYNCH_GET_ST, END_SYNCH_GET_ST|BEG_SYNCH_SEND_CALLS, END_SYNCH_SEND_CALLS| lambda
rest_seq2 -> GW_SEND_REQUESTS, act_repl_s, RECEIVE_FIRST_REPLY, EXIT_RECEIVE_REPLY
act_repl_s -> SEND_REQ_EXP_EXC|EXIT_RECEIVE_REQUEST, EXIT_SEND_REQUEST

*/

/** Sample Compiler class used to process trace information written in the appropriate files while requests are travelling from client to server and back
  * The idea is to take those files, in which information about starting and ending of certain phases in the request travelling are traced and parse them.
  * The informations writtne there are of the form request client identifier, request retention identifier, moment(phase) code and start time for that phase.
  * For example, one moment is the entering of the server request in cold passive replication: thus, a record written in such a file can contain:
  *  2(constant coresponding to ENTER_RECEIVE_REQUEST)
  *  234 (Retention identifier of the request)
  *  0*ORB@123lkdiw8222 (Client identifier of the request)
  *  124985436578435 (the start time of the phase, in milliseconds since jan 1st 1970 - as returned by System.currentTimeMillis())
  * All trace files are read and these types of records are collected and inserted in a hashtable containing as keys the unique request identifiers (client_id+
  * retention_id). A vector containing all moment records for a certain request is sorted according to time. The data about requests (such as how long a certain
  * operation lasted - e.g. how long the request spent in the server interceptor) are written in files corresponding to every method name, and request phase name.
  * For example, if a method with name "add_numbers" was called 100 times during the experiment then there are 100 values to be written in the file add_numbers_SCL,
  * 100 values to be written in the file add_numbers_TTS, 100 values to be written in the file add_numbers_ASP, etc. The file add_numbers_SCL contains time intervals
  * that the request spent in the client interceptor before travelling to the server, in all the hundred occasions when the method add_numbers was called. Of course,
  * the 100 values correspond to 100 requests with different retention identifiers and perhaps the same client identifier, but to the same name of the method that was
  * called. There is also a file corresponding to total roundtrip times. The results that are provided as statistics are obtained by averaging the e.g. 100 values from the
  * files. The version with writing the set of timing values to the specific files is implemented only for COLD and WARM PASSIVE replicastion; in case of active
  * replication, the numbers are summed and a final average is output from the execution of the compiler.
  * NOTE: the implementation of the Compiler can be changed so that it fits trace processing needs, as desired. For, example, other traces can be introduced
  * and these need other processing steps than present in the Compiler's present implementation
   * @author <a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
   * @version $Revision: 1.0 $ $Date:2003/06/23$
 */

public class SampleTraceCompiler{

    final static int NR_CHARS_OPER=8;
    final static int NR_DIGITS_NR=5;


    private class FailOverT{
	int nr_f_o_ocr;
	int nr_f_o_nocr;
	int sum_oc;
	int sum_noc;

	public FailOverT(){
	    nr_f_o_ocr=0;
	    nr_f_o_nocr=0;
	    sum_oc=0;
	    sum_noc=0;
	}
    }

 private class ReplMTrStr{
	int moment_code;
	int crt_nr_replicas;
	int repl_style;
	long time;
	public ReplMTrStr(int moment_code,int crt_nr_replicas,int repl_style,long time){
	    this.moment_code=moment_code;
	    this.crt_nr_replicas=crt_nr_replicas;
	    this.repl_style=repl_style;
	    this.time=time;
	}
    }
    private class ReqTraceStr{

	int moment_code,replica_nr;
	int repl_style;
	int retention_id;
	String client_id;
	long time;
	String oper_name;


	public ReqTraceStr(int replica_nr,int moment_code,int repl_style,int retention_id,String client_id,long time,String oper_name){
	    this.replica_nr=replica_nr;
	    this.moment_code=moment_code;
	    this.repl_style=repl_style;
	    this.time=time;
	    this.client_id=client_id;
	    this.retention_id=retention_id;
	    this.oper_name=oper_name;
	}
    }

    private class GetStateTrStr{
	int moment_code;
	int size;
	long time;

	public GetStateTrStr(int moment_code,int size,long time){
	    this.moment_code=moment_code;
	    this.size=size;
	    this.time=time;
	}
    }

    private class AvRecStr{
	long sum,sum_sq,sum_sq_without_wait,sum_without_wait;
	long max_value,max_value_without_wait;
	int nr_of_calls;

	public AvRecStr(){
	    max_value=-1;
	    max_value_without_wait=-1;
	    sum_sq=0;
	    sum_without_wait=0;
	    sum_sq_without_wait=0;
	    sum=0;
	    nr_of_calls=0;
	}
    }

    private class AvWaitRecStr{
	long sum,sum_sq;
	int nr_of_calls;

	public AvWaitRecStr(){
	    sum=0;
	    nr_of_calls=0;
	}
    }

    private class AvSliceRecStr{
	long sum,sum_sq,max_value;
	int nr_of_calls;
	String oper;

	public AvSliceRecStr(String oper){
	    nr_of_calls=0;
	    max_value=-1;
	    sum=0;
	    this.oper=oper;
	}
    }

/** Method used to read a "line" from a request tracing file; this line contains retention identifier of a request (in int format), moment_code (in int format),
* time (in long format) and client identifier and operation name (as strings separated wit space)
*/

    private void read_req_t_line(java.io.RandomAccessFile raf,int repl_style){
	int moment_code,not_normal;
	int retention_id,replica_nr=-1;
	String client_id,hashcode,oper_name;
	long time;
	boolean put=false;
	try{
	    not_normal=raf.readInt();
	    if(not_normal==1)
		replica_nr=raf.readInt();
	    moment_code=raf.readInt();
	    retention_id=raf.readInt();
	    time=raf.readLong();
	    client_id=raf.readLine();
	    oper_name=raf.readLine();
	    hashcode=client_id+"+"+oper_name+"+"+retention_id;
	    java.util.Vector req_trace_v=(java.util.Vector)req_trace_t.get(hashcode);
	    if(req_trace_v==null){
		put=true;
		req_trace_v=new java.util.Vector();
	    }
	    if(repl_style!=3&&req_trace_v.size()>36) return;
	    req_trace_v.addElement(new ReqTraceStr(replica_nr,moment_code,repl_style,retention_id,client_id,time,oper_name));
	    if(put){
		req_trace_t.put(hashcode,req_trace_v);
	    }
	}catch(java.lang.Exception e){e.printStackTrace();}
    }

    private void read_repl_m_t_line(java.io.RandomAccessFile raf){
	int moment_code;
	long group_id;
	int crt_nr_replicas;
	int repl_style;
	long time;
	boolean put=false;
	try{
	    moment_code=raf.readInt();
	    group_id=raf.readLong();
	    crt_nr_replicas=raf.readInt();
	    repl_style=raf.readInt();
	    time=raf.readLong();
	    java.util.Vector repl_m_trace_for_gr=(java.util.Vector)repl_m_trace_t.get(new Long(group_id));
	    if(repl_m_trace_for_gr==null){
		put=true;
		repl_m_trace_for_gr=new java.util.Vector();
	    }
	    repl_m_trace_for_gr.addElement(new ReplMTrStr(moment_code,crt_nr_replicas,repl_style,time));
	    if(put)
		repl_m_trace_t.put(new Long(group_id),repl_m_trace_for_gr);
	}catch(java.lang.Exception e){e.printStackTrace();}
    }

    private void read_get_s_t_line(java.io.RandomAccessFile raf){
	int moment_code;
	int state_size;
	long time,obj_id;
	boolean put=false;
	java.util.Vector get_state_trace_v;
	try{
	    moment_code=raf.readInt();
	    state_size=raf.readInt();
	    obj_id=raf.readLong();
	    time=raf.readLong();
	    get_state_trace_v=(java.util.Vector)get_state_trace_t.get(new Long(obj_id));
	    if(get_state_trace_v==null){
		get_state_trace_v=new java.util.Vector();
		get_state_trace_t.put(new Long(obj_id),get_state_trace_v);
	    }
	    get_state_trace_v.addElement(new GetStateTrStr(moment_code,state_size,time));
	}catch(java.lang.Exception e){e.printStackTrace();}
    }


    private void switch_elements(java.util.Vector v,java.lang.Object obj,int pos){
	v.addElement(obj);
	v.remove(pos);
    }

    private boolean is_not_ok(int l,boolean[] b_v_s){
	int i;
	boolean ret=false;
	for(i=0;i<l;i++)
	    ret=ret||!b_v_s[i];
	return ret;
    }

    private AvSliceRecStr get_slice(String oper,java.util.Vector slices){
	int l=slices.size(),i;
	AvSliceRecStr asrs;
	for(i=2;i<l;i++){
	    asrs=(AvSliceRecStr)slices.elementAt(i);
	    if(asrs.oper.equals(oper))
		return asrs;
	}
	asrs=new AvSliceRecStr(oper);
	slices.addElement(asrs);
	return asrs;
    }

    private void print_vector(java.util.Vector v){
	int l=v.size(),i;

	for(i=0;i<l;i++)
	    System.out.print(((ReqTraceStr)v.elementAt(i)).moment_code+" "+((ReqTraceStr)v.elementAt(i)).time+"  ");
	System.out.println();
    }

    private String pad_str(String s,int nl){
	int i,l;
	String toRet="";
	l=s.length();
	if(l==nl) return s;
	else{
	    for(i=0;i<(nl-l)/2;i++)
		toRet+=" ";
	    toRet+=s;
	    for(i=0;i<nl-l-(nl-l)/2;i++)
		toRet+=" ";
	    return toRet;
	}
    }

    private String pad_nr(int nr,int nl){

	String toRet=Integer.toString(nr);

	if(toRet.length()==nl) return toRet;
	StringBuffer toRetf=new StringBuffer();
	for(int i=0;i<nl-toRet.length();i++)
	    toRetf.append(" ");
	toRetf.append(toRet);
	return toRetf.toString();
    }

    private void sort_g_s_vector_t(java.util.Vector v){
	int i,l,j;
	GetStateTrStr crt,next;

	l=v.size();
	for(i=0;i<l;i++)
	    for(j=0;j<l-i-1;j++){
		crt=(GetStateTrStr)v.elementAt(j);
		next=(GetStateTrStr)v.elementAt(j+1);
		if(crt.time>next.time){
		    if(j<l-2){
			v.insertElementAt(crt,j+2);
			v.remove(j);
		    }
		    else
			{
			    v.insertElementAt(next,j);
			    v.remove(l);
			}
		}
	    }
    }


    private void sort_vector_t(java.util.Vector v){
	int i,l,j;
	ReqTraceStr crt,next;

	l=v.size();
	for(i=0;i<l;i++)
	    for(j=0;j<l-i-1;j++){
		crt=(ReqTraceStr)v.elementAt(j);
		next=(ReqTraceStr)v.elementAt(j+1);
		if(crt.time>next.time){
		    if(j<l-2){
			v.insertElementAt(crt,j+2);
			v.remove(j);
		    }
		    else
			{
			    v.insertElementAt(next,j);
			    v.remove(l);
			}
		}
	    }
}

    public SampleTraceCompiler(){
	repl_m_trace_t=new java.util.Hashtable();
	req_trace_t=new java.util.Hashtable();
	oper_av_t=new java.util.Hashtable();
	oper_av_w_t=new java.util.Hashtable();
	times=new java.util.Hashtable();
	get_state_trace_t=new java.util.Hashtable();
    }

    java.util.Hashtable repl_m_trace_t;
    java.util.Hashtable req_trace_t,times;
    static java.util.Hashtable oper_av_t,oper_av_w_t,get_state_trace_t;
    static int current_repl_style;

    public void parse_repl_m_trace(String f_name){
	java.io.RandomAccessFile repl_m_trace;
	try{
	    repl_m_trace=new java.io.RandomAccessFile(f_name,"rw");
	    long length=repl_m_trace.length();
	    while(repl_m_trace.getFilePointer()<length){
		read_repl_m_t_line(repl_m_trace);
	    }
	    repl_m_trace.close();
	}catch(java.lang.Exception e){e.printStackTrace();}
    }

    public void parse_req_trace(String f_name){
	java.io.RandomAccessFile req_trace;
	boolean error=false;
	try{
	    req_trace=new java.io.RandomAccessFile(f_name,"rw");
	    long length=req_trace.length();
	    int repl_style=1,i;
	    i=f_name.lastIndexOf("%");
	    if(f_name.substring(i+1).equals("gw")){
		repl_style=org.omg.FT.ACTIVE.value;
		current_repl_style=repl_style;
	    }
	    else{
		try{
		    Integer.parseInt(f_name.substring(0,1));
		}catch(java.lang.Exception e){error=true;}
		if(!error){
		    try{
			repl_style=Integer.parseInt(f_name.substring(i-2,i-1));
			current_repl_style=repl_style;
		    }catch(java.lang.Exception e){e.printStackTrace();}
		}
		else
		    repl_style=current_repl_style;
	    }
	    while(req_trace.getFilePointer()<length){
		read_req_t_line(req_trace,repl_style);
	    }
	    req_trace.close();
	}catch(java.lang.Exception e){}
    }

    private String spaces(int nr){
	String t_r="";
	int i;
	for(i=0;i<nr;i++)
	    t_r+=" ";
	return t_r;
    }

    public void parse_get_state_trace(String f_name){
	java.io.RandomAccessFile get_s_trace;
	try{
	    get_s_trace=new java.io.RandomAccessFile(f_name,"rw");
	    long length=get_s_trace.length();
	    while(get_s_trace.getFilePointer()<length){
		read_get_s_t_line(get_s_trace);
	    }
	    get_s_trace.close();
	}catch(java.lang.Exception e){}
    }


    public void compute_req_times(String experim_name){
	/* "operations" for cold-warm passive replication style, and part of them for stateless

	   SCL - stay in client interceptor
	   TTS - travel to server
	   SUSFE - stay until send forward exception
	      or
	   SUSWP - stay until send wrong primary exception
	      or
	   SUSREE - stay until send request expired exception
	   TUDRRP - time until discover that request is at the right place
	   TUFSR - time until find and send (if found) reply
	   TUSGS - time until start synchr. get_state
	   TSGS - time while synchronize with get_state
	   TUERR - time until exit receive_request method
	   ASP - approximate time for server (application) processing
	   NLSP - time which passes from the exit receive_request because the reply is there, until the enter
	   send_reply, which DOES NOT include server processing; it should be smaller than ASP
	   TSR - time spent in send reply
	   TERR - time spent on travelling back and in the receive_reply method of the client interc.
	   or
	   TRE - time spent to travel back and exit the receive_exception method of the client interc.
	   or
	   TRO - time spent to travel back and exit the receive_other method of the client interc.

	   TSE - time until send exception
	   TSO - time until send other

	   TTGE - time to travel from server interc to client when announcing request expired already at gateway
	   TTAE - time to travel from gateway when request expired at the active replica
	   TEGI - time to travel from client to gw. and enter gateway interceptor
	   TAGNA - time to announce that group is not available

	*/


	try{
	    java.io.RandomAccessFile table=new java.io.RandomAccessFile(experim_name,"rw");
	    table.writeBytes("R. S.|Ret.Id.|                           Client Id.                      |Operation|Time\n");
	    table.writeBytes("---------------------------------------------------------------------------------------\n\n");
	    int size,i,repl_style,j;
	    ReqTraceStr stat_elem;
	    java.util.Set entrySet=req_trace_t.entrySet();
	    java.util.Iterator iter=entrySet.iterator();
	    java.util.Map.Entry entry;
	    java.util.Vector ret_cl_id_vector;
	    boolean possible_paths[],put;
	    int retention_id,replica;
	    String client_id;
	    long rtt_without_wait=0,time_enter_send_request=0,time_exit_send_request=0,time_enter_receive_request=0,time_send_fwd_exc=0,time_send_wr_prim_exc=0,time_send_req_exp_exc=0,time_receive_exception=0,time_receive_other=0,time_yes_req_right_place=0,time_exit_reply_there=0,time_exit_receive_request=0,time_enter_send_reply=0,time_exit_send_reply=0,time_exit_send_reply_gw=0,time_exit_receive_reply=0, time_receive_first_reply=0, time_gw_send_requests=0, time_beg_synch_failover=0,time_end_synch_failover=0,time_begin_synch_send_calls=0,time_end_synch_send_calls=0,send_req_exp_exc_gw=0, time_gw_send_transient=0,time_send_req_exp_exc_gw=0, time_enter_receive_request_gw=0,time_send_other=0,time_send_exception=0,time_send_exception_rm=0,time_send_other_rm=0,time_start_rec_method_call=0,time_start_rec_reply=0,time_start_rec_br_method_call=0,time_start_rec_br_reply=0,time_start_exec_meth=0,time_stop_exec_meth=0,time_stop_rec_method_call=0,time_stop_rec_reply=0,time_stop_rec_br_method_call=0,time_stop_rec_br_reply=0,interval,total_time_crt,total_time_w,time_stop_wait_upd_oper,time_start_wait_upd_oper;
	    AvRecStr av_str;
	    java.util.Vector avg_vector;
	    AvWaitRecStr av_w_str;
	    AvSliceRecStr av_sl_str;
	    boolean decr;
	    java.util.Vector[] vectors;

	    while(iter.hasNext()){
		rtt_without_wait=0;
		time_stop_wait_upd_oper=0;
		time_start_wait_upd_oper=0;
		time_enter_send_request=0;
		time_exit_send_request=0;
		time_enter_receive_request=0;
		time_send_fwd_exc=0;
		time_send_wr_prim_exc=0;
		time_send_req_exp_exc=0;
		time_receive_exception=0;
		time_receive_other=0;
		time_yes_req_right_place=0;
		time_exit_reply_there=0;
		time_exit_receive_request=0;
		time_enter_send_reply=0;
		time_exit_send_reply=0;
		time_exit_receive_reply=0;
		time_receive_first_reply=0;
		time_gw_send_requests=0;
		time_beg_synch_failover=0;
		time_end_synch_failover=0;
		time_begin_synch_send_calls=0;
		time_end_synch_send_calls=0;
		send_req_exp_exc_gw=0;
		time_gw_send_transient=0;
		time_send_req_exp_exc_gw=0;
		time_enter_receive_request_gw=0;
		time_send_other=0;
		time_send_exception=0;
		time_exit_send_reply_gw=0;
		time_send_exception_rm=0;
		time_send_other_rm=0;
		time_start_rec_method_call=0;
		time_start_rec_reply=0;
		time_start_rec_br_method_call=0;
		time_start_rec_br_reply=0;
		time_stop_rec_method_call=0;
		time_stop_rec_reply=0;
		time_stop_rec_br_method_call=0;
		time_stop_rec_br_reply=0;
		time_start_exec_meth=0;
		time_stop_exec_meth=0;
		interval=0;
		table.writeBytes("\n");
		entry=(java.util.Map.Entry)iter.next();
		ret_cl_id_vector=(java.util.Vector)entry.getValue();
		size=ret_cl_id_vector.size();
		if(size==0) continue;
		repl_style=((ReqTraceStr)ret_cl_id_vector.elementAt(0)).repl_style;
		retention_id=((ReqTraceStr)ret_cl_id_vector.elementAt(0)).retention_id;
		client_id=((ReqTraceStr)ret_cl_id_vector.elementAt(0)).client_id;
		String oper_name;
		oper_name=((ReqTraceStr)ret_cl_id_vector.elementAt(0)).oper_name;
		if(oper_name==null)
		    continue;
		vectors=(java.util.Vector[])times.get(oper_name);
		if(vectors==null){
		    vectors=new java.util.Vector[23];
		    times.put(oper_name,vectors);
		    for(i=0;i<23;i++)
			vectors[i]=new java.util.Vector();
		}
		av_str=null;
		avg_vector=null;
		try{
		    avg_vector=(java.util.Vector)oper_av_t.get(oper_name);
		}catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));}


		put=false;
		if(avg_vector==null) {
		    avg_vector=new java.util.Vector();
		    av_w_str=new AvWaitRecStr();
		    av_str=new AvRecStr();
		    put=true;
		    avg_vector.addElement(av_str);
		    avg_vector.addElement(av_w_str);
		}
		else{
		    av_str=(AvRecStr)avg_vector.elementAt(0);
		    av_w_str=(AvWaitRecStr)avg_vector.elementAt(1);
		}
		if(retention_id==-1) continue;
		total_time_crt=av_str.sum;
		total_time_w=av_w_str.sum;
		av_str.nr_of_calls++;
		av_w_str.nr_of_calls++;
		table.writeBytes(org.omg.FT.Utils.repl_s_string(repl_style)+"|"+pad_nr(retention_id,NR_DIGITS_NR)+"|"+client_id+"|"+pad_str("",NR_CHARS_OPER)+"|"+oper_name+"\n\n");

		if(size>35&&repl_style!=3) continue;
		sort_vector_t(ret_cl_id_vector);
		decr=false;
		switch(repl_style){
		case org.omg.FT.STATELESS.value:
		    possible_paths=new boolean[5];
		    for(i=0;i<5;i++)
			possible_paths[i]=false;

		    for(i=0;i<size;i++){
			stat_elem=(ReqTraceStr)ret_cl_id_vector.elementAt(i);
			switch(stat_elem.moment_code){
			case org.omg.FT.Utils.ENTER_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_send_request=stat_elem.time;
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_exit_send_request=stat_elem.time;
			    if(time_enter_send_request!=0)
				interval=time_exit_send_request-time_enter_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SCL",NR_CHARS_OPER)+"|"+interval+"\n");

			    break;
			case org.omg.FT.Utils.ENTER_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_receive_request=stat_elem.time;
			    if(time_exit_send_request!=0)
				interval=time_enter_receive_request-time_exit_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes(org.omg.FT.Utils.repl_s_string(repl_style)+"|"+pad_nr(stat_elem.retention_id,NR_DIGITS_NR)+"|"+stat_elem.client_id+"|"+pad_str("TTS",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_FORWARD_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=0)
				    possible_paths[j]=false;
			    time_send_fwd_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_fwd_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSFE",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_WR_PRIM_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=1)
				    possible_paths[j]=false;
			    time_send_wr_prim_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_wr_prim_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSWPE",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_REQ_EXP_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=2)
				    possible_paths[j]=false;
			    time_send_req_exp_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_req_exp_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSREE",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.YES_REQUEST_RIGHT_PLACE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3&&j!=4)
				    possible_paths[j]=false;
			    time_yes_req_right_place=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_yes_req_right_place-time_enter_receive_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUDRRP",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.EXIT_REPLY_ALREADY_THERE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3)
				    possible_paths[j]=false;
			    time_exit_reply_there=stat_elem.time;
			    if(time_yes_req_right_place!=0)
				interval=time_exit_reply_there-time_yes_req_right_place;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUFSR",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=4)
				    possible_paths[j]=false;
			    time_exit_receive_request=stat_elem.time;
			    if(time_yes_req_right_place!=0)
				interval=time_exit_receive_request-time_yes_req_right_place;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.ENTER_SEND_REPLY:

			    time_enter_send_reply=stat_elem.time;
			    if(time_exit_receive_request!=0)
				interval=time_enter_send_reply-time_exit_receive_request;
			    else
				interval=0;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("ASP",NR_CHARS_OPER)+"|"+interval+"\n");
			    av_str.sum+=interval;//For now
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REPLY:
			    time_exit_send_reply=stat_elem.time;
			    if(time_enter_send_reply!=0)
				interval=time_exit_send_reply-time_enter_send_reply;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSR",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REPLY:
			    time_exit_receive_reply=stat_elem.time;
			    if(possible_paths[3])
				if(time_exit_reply_there!=0)
				    interval=time_exit_receive_reply-time_exit_reply_there;
				else
				    interval=0;
			    else
				if(time_exit_send_reply!=0)
				    interval=time_exit_receive_reply-time_exit_send_reply;
				else
				    interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_OTHER:
			     time_send_other=stat_elem.time;
			     if(possible_paths[4])
				 if(time_exit_receive_request!=0)
				     interval=time_send_other-time_exit_receive_request;
				 else
				     interval=time_send_other-time_exit_send_reply;
			     else interval=0;
			     av_str.sum+=interval;
			     table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSO",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION:
			    time_send_exception=stat_elem.time;
			    if(possible_paths[4])
				if(time_exit_receive_request!=0)
				    interval=time_send_exception-time_exit_receive_request;
				else
				    interval=time_send_exception-time_exit_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSE",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_OTHER_RM:
			    time_send_other_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION_RM:
			    time_send_exception_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.RECEIVE_OTHER:
			    time_receive_other=stat_elem.time;
			    if(time_send_other_rm!=0){
				interval=time_receive_other-time_send_other_rm;
				if(!decr){
				    av_str.nr_of_calls--;
				    av_w_str.nr_of_calls--;
				    decr=true;
				}
				av_str.sum=total_time_crt;
				av_w_str.sum=total_time_w;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRORM",NR_CHARS_OPER)+"|"+interval+"\n");
			    }
			    else
				if(time_send_other!=0){
				    interval=time_receive_other-time_send_other;
				    if(!decr){
					av_str.nr_of_calls--;
					av_w_str.nr_of_calls--;
					decr=true;
				    }
				    av_str.sum=total_time_crt;
				    av_w_str.sum=total_time_w;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
				}
			    	else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_other-time_send_req_exp_exc;
					if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_other-time_send_wr_prim_exc;
					    if(!decr){
						av_w_str.nr_of_calls--;
						av_str.nr_of_calls--;
						decr=true;
					    }
					    av_w_str.sum=total_time_w;
					    av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_other-time_send_fwd_exc;
						if(!decr){
						    av_str.nr_of_calls--;
						    av_w_str.nr_of_calls--;
						    decr=true;
						}
						av_str.sum=total_time_crt;
						av_w_str.sum=total_time_w;
						table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_other-time_exit_send_request;
						    if(!decr){
							av_str.nr_of_calls--;
							av_w_str.nr_of_calls--;
							decr=true;
						    }
						    av_str.sum=total_time_crt;
						    av_w_str.sum=total_time_w;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						}
			    break;
			case org.omg.FT.Utils.RECEIVE_EXCEPTION:
			    time_receive_exception=stat_elem.time;
			    if(time_send_exception_rm!=0){
				interval=time_receive_exception-time_send_exception_rm;
				  if(!decr){
				      av_str.nr_of_calls--;
				      av_w_str.nr_of_calls--;
				      decr=true;
				  }
				  av_w_str.sum=total_time_w;
				  av_str.sum=total_time_crt;
				//av_str.sum+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRERM",NR_CHARS_OPER)+"|"+interval+"\n");
			    }
			    else
				if(time_send_exception!=0){
				    interval=time_receive_exception-time_send_exception;
				    if(!decr){
					av_w_str.nr_of_calls--;
					av_str.nr_of_calls--;
					decr=true;
				    }
				    av_w_str.sum=total_time_w;
				    av_str.sum=total_time_crt;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
				}
			    	else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_exception-time_send_req_exp_exc;
					if(!decr){
					    av_w_str.nr_of_calls--;
					    av_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_exception-time_send_wr_prim_exc;
						if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_exception-time_send_fwd_exc;
						if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
						table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_exception-time_exit_send_request;
						    	if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						}
			    break;
			}

		    }
		    break;
		case org.omg.FT.COLD_PASSIVE.value:
		    possible_paths=new boolean[5];
		    for(i=0;i<5;i++)
			possible_paths[i]=false;

		    for(i=0;i<size;i++){
			stat_elem=(ReqTraceStr)ret_cl_id_vector.elementAt(i);
			switch(stat_elem.moment_code){
			case org.omg.FT.Utils.START_WAIT_UPD_OPER:
			    time_start_wait_upd_oper=stat_elem.time;
			    break;
			case org.omg.FT.Utils.STOP_WAIT_UPD_OPER:
			    time_stop_wait_upd_oper=stat_elem.time;
			     if(time_start_wait_upd_oper!=0)
				 interval=time_stop_wait_upd_oper-time_start_wait_upd_oper;
			     else
				 interval=0;
			     av_w_str.sum+=interval;
			     av_w_str.sum_sq+=interval*interval;
			     av_str.sum+=interval;
			     vectors[22].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.ENTER_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_send_request=stat_elem.time;
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_exit_send_request=stat_elem.time;
			    if(time_enter_send_request!=0)
				interval=time_exit_send_request-time_enter_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("SCL",avg_vector);
			    if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    // av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    // av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SCL",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[0].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.ENTER_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_receive_request=stat_elem.time;
			    if(time_exit_send_request!=0)
				interval=time_enter_receive_request-time_exit_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("TTS",avg_vector);
			    if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    //av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTS",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[1].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_FORWARD_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=0)
				    possible_paths[j]=false;
			    time_send_fwd_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_fwd_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("SUSFE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    //av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSFE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[2].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_WR_PRIM_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=1)
				    possible_paths[j]=false;
			    time_send_wr_prim_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_wr_prim_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("SUSWPE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    //av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSWPE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[3].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_REQ_EXP_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=2)
				    possible_paths[j]=false;
			    time_send_req_exp_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_req_exp_exc-time_enter_receive_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("SUSREE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    // av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSREE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[4].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.YES_REQUEST_RIGHT_PLACE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3&&j!=4)
				    possible_paths[j]=false;
			    time_yes_req_right_place=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_yes_req_right_place-time_enter_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("TUDRRP",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    //av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUDRRP",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[5].addElement(new Long(interval));
			    break;

			case org.omg.FT.Utils.START_REC_METHOD_CALL:
			    time_start_rec_method_call=stat_elem.time;
			    if(time_yes_req_right_place!=0)
				interval=time_start_rec_method_call-time_yes_req_right_place;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("TUSRMC",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    // av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUSRMC",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[6].addElement(new Long(interval));
			    /* TUSRMC - time until start record method call in cold passive*/
			    break;
			case org.omg.FT.Utils.STOP_REC_METHOD_CALL:
			    time_stop_rec_method_call=stat_elem.time;
			    if(time_start_rec_method_call!=0)
				interval=time_stop_rec_method_call-time_start_rec_method_call;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("TRMC",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    //av_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRMC",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[7].addElement(new Long(interval));
			    /* TRMC - time to record method call in cold passive*/
			    break;
			case org.omg.FT.Utils.EXIT_REPLY_ALREADY_THERE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3)
				    possible_paths[j]=false;
			    time_exit_reply_there=stat_elem.time;
			    if(time_stop_rec_method_call!=0)
				interval=time_exit_reply_there-time_stop_rec_method_call;
			    else
				interval=0;
			    av_sl_str=get_slice("TUFSR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUFSR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[8].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=4)
				    possible_paths[j]=false;
			    time_exit_receive_request=stat_elem.time;
			    if(time_stop_rec_method_call!=0)
				interval=time_exit_receive_request-time_stop_rec_method_call;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("TUERR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[9].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.ENTER_SEND_REPLY:
			    time_enter_send_reply=stat_elem.time;
			    if(time_exit_receive_request!=0)
				interval=time_enter_send_reply-time_exit_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("ASP",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;//For now
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("ASP",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[10].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.START_REC_REPLY:
			    time_start_rec_reply=stat_elem.time;
			    if(time_enter_send_reply!=0)
				interval=time_start_rec_reply-time_enter_send_reply;
			    else
				interval=0;
			    av_sl_str=get_slice("TESRRR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TESRRR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[11].addElement(new Long(interval));
			    /* TESRRR - time from enter send reply to record reply*/
			    break;
			case org.omg.FT.Utils.STOP_REC_REPLY:
			    time_stop_rec_reply=stat_elem.time;
			    if(time_start_rec_reply!=0)
				interval=time_stop_rec_reply-time_start_rec_reply;
			    else
				interval=0;
			    av_sl_str=get_slice("TRR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[12].addElement(new Long(interval));
			    /* TRR time to record reply (without broadcast in COLD_PASSIVE)*/
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REPLY:
			    time_exit_send_reply=stat_elem.time;
			    if(time_stop_rec_reply!=0){
				interval=time_exit_send_reply-time_stop_rec_reply;
				av_sl_str=get_slice("TRRESR",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
				av_str.sum+=interval;
				av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRRESR",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[13].addElement(new Long(interval));
				/*TRRESR  Time from end record reply to exit send reply*/
			    }
			    else{
				if(time_enter_send_reply!=0)
				    interval=time_exit_send_reply-time_enter_send_reply;
				else
				    interval=0;
				av_sl_str=get_slice("TSR",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
				av_str.sum+=interval;
				av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSR",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[14].addElement(new Long(interval));
			    }
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REPLY:
			    time_exit_receive_reply=stat_elem.time;
			    if(possible_paths[3])
				if(time_exit_reply_there!=0)
				    interval=time_exit_receive_reply-time_exit_reply_there;
				else
				    interval=0;
			    else
				if(time_exit_send_reply!=0)
				    interval=time_exit_receive_reply-time_exit_send_reply;
				else
				    interval=0;
			    av_sl_str=get_slice("TERR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[15].addElement(new Long(interval));
			    break;

			case org.omg.FT.Utils.SEND_OTHER:
			    time_send_other=stat_elem.time;
			    if(possible_paths[4])
				if(time_exit_receive_request!=0)
				    interval=time_send_other-time_exit_receive_request;
				else
				    interval=time_send_other-time_exit_send_request;
			    else interval=0;
			    av_sl_str=get_slice("TSO",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSO",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[16].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION:
			    time_send_exception=stat_elem.time;
			    if(possible_paths[4])
				if(time_exit_receive_request!=0)
				    interval=time_send_exception-time_exit_receive_request;
				else
				    interval=time_send_exception-time_exit_send_request;
			    else
				interval=0;
			    av_str.sum+=interval;
			    av_sl_str=get_slice("TSE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[17].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_OTHER_RM:
			    time_send_other_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION_RM:
			    time_send_exception_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.RECEIVE_OTHER:
			    time_receive_other=stat_elem.time;
			    if(time_send_other_rm!=0){
				interval=time_receive_other-time_send_other_rm;
				  if(!decr){
				      av_str.nr_of_calls--;
				      av_w_str.nr_of_calls--;
				      decr=true;
				  }
				av_str.sum=total_time_crt;
				av_w_str.sum=total_time_w;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRORM",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[18].addElement(new Long(interval));
			    }
			    else
				if(time_send_other!=0){
				    interval=time_receive_other-time_send_other;
				    if(!decr){
					av_str.nr_of_calls--;
					av_w_str.nr_of_calls--;
					decr=true;
				    }
				    av_w_str.sum=total_time_w;
				    av_str.sum=total_time_crt;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
				    vectors[19].addElement(new Long(interval));
				}
				else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_other-time_send_req_exp_exc;
					if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					vectors[19].addElement(new Long(interval));
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_other-time_send_wr_prim_exc;
					    if(!decr){
						av_str.nr_of_calls--;
						av_w_str.nr_of_calls--;
						decr=true;
					    }
					    av_w_str.sum=total_time_w;
					    av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					    vectors[19].addElement(new Long(interval));
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_other-time_send_fwd_exc;
						if(!decr){
						    av_str.nr_of_calls--;
						    av_w_str.nr_of_calls--;
						    decr=true;
						}
						av_w_str.sum=total_time_w;
						av_str.sum=total_time_crt;
						table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
						vectors[19].addElement(new Long(interval));
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_other-time_exit_send_request;
						    if(!decr){
							av_str.nr_of_calls--;
							av_w_str.nr_of_calls--;
							decr=true;
						    }
						    av_w_str.sum=total_time_w;
						    av_str.sum=total_time_crt;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						    vectors[20].addElement(new Long(interval));
						}
			    break;
			case org.omg.FT.Utils.RECEIVE_EXCEPTION:
			    time_receive_exception=stat_elem.time;
			    if(time_send_exception_rm!=0){
				interval=time_receive_exception-time_send_exception_rm;
				if(!decr){
				    av_str.nr_of_calls--;
				    av_w_str.nr_of_calls--;
				    decr=true;
				}
				av_w_str.sum=total_time_w;
				av_str.sum=total_time_crt;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRERM",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[21].addElement(new Long(interval));
			    }
			    else
				if(time_send_exception!=0){
				    interval=time_receive_exception-time_send_exception;
				    if(!decr){
					av_str.nr_of_calls--;
					av_w_str.nr_of_calls--;
					decr=true;
				    }
				    av_w_str.sum=total_time_w;
				    av_str.sum=total_time_crt;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
				    vectors[20].addElement(new Long(interval));
				}
				else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_exception-time_send_req_exp_exc;
				  if(!decr){
				      av_str.nr_of_calls--;
				      av_w_str.nr_of_calls--;
				      decr=true;
				  }
				av_str.sum=total_time_crt;
				av_w_str.sum=total_time_w;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[20].addElement(new Long(interval));
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_exception-time_send_wr_prim_exc;
					    if(!decr){
						av_str.nr_of_calls--;
						av_w_str.nr_of_calls--;
						decr=true;
					    }
					    av_w_str.sum=total_time_w;
					    av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					    vectors[20].addElement(new Long(interval));
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_exception-time_send_fwd_exc;
						if(!decr){
						    av_str.nr_of_calls--;
						    av_w_str.nr_of_calls--;
						    decr=true;
						}
						av_w_str.sum=total_time_w;
						av_str.sum=total_time_crt;
						table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						vectors[20].addElement(new Long(interval));
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_exception-time_exit_send_request;
						    if(!decr){
							av_str.nr_of_calls--;
							av_w_str.nr_of_calls--;
							decr=true;
						    }
						    av_w_str.sum=total_time_w;
						    av_str.sum=total_time_crt;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						    vectors[20].addElement(new Long(interval));
						}
			    break;
			}
		    }
		    break;
		case org.omg.FT.WARM_PASSIVE.value:
		    possible_paths=new boolean[5];
		    for(i=0;i<5;i++)
			possible_paths[i]=false;

		    for(i=0;i<size;i++){
			stat_elem=(ReqTraceStr)ret_cl_id_vector.elementAt(i);
			switch(stat_elem.moment_code){
			case org.omg.FT.Utils.START_WAIT_UPD_OPER:
			    time_start_wait_upd_oper=stat_elem.time;
			    break;
			case org.omg.FT.Utils.STOP_WAIT_UPD_OPER:
			    time_stop_wait_upd_oper=stat_elem.time;
			    if(time_start_wait_upd_oper!=0){
				interval=time_stop_wait_upd_oper-time_start_wait_upd_oper;
				av_w_str.sum+=interval;
				av_w_str.sum_sq+=interval*interval;
				av_str.sum+=interval;
			    }
			    else
				interval=0;
			    vectors[0].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.ENTER_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_send_request=stat_elem.time;
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_exit_send_request=stat_elem.time;
			    if(time_enter_send_request!=0)
				interval=time_exit_send_request-time_enter_send_request;
			    else
				interval=0;
			    av_sl_str=get_slice("SCL",avg_vector);
			    if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    av_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SCL",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[1].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.ENTER_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_receive_request=stat_elem.time;
			    if(time_exit_send_request!=0)
				interval=time_enter_receive_request-time_exit_send_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TTS",avg_vector);
  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTS",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[2].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_FORWARD_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=0)
				    possible_paths[j]=false;
			    time_send_fwd_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_fwd_exc-time_enter_receive_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("SUSFE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSFE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[3].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_WR_PRIM_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=1)
				    possible_paths[j]=false;
			    time_send_wr_prim_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_wr_prim_exc-time_enter_receive_request;
			    else{
				interval=0;
			    }
			    	av_sl_str=get_slice("SUSWPE",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSWPE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[4].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_REQ_EXP_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=2)
				    possible_paths[j]=false;
			    time_send_req_exp_exc=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_send_req_exp_exc-time_enter_receive_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("SUSREE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SUSREE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[5].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.YES_REQUEST_RIGHT_PLACE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3&&j!=4)
				    possible_paths[j]=false;
			    time_yes_req_right_place=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_yes_req_right_place-time_enter_receive_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TUDRRP",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUDRRP",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[6].addElement(new Long(interval));
			    break;

			case org.omg.FT.Utils.START_REC_BR_METHOD_CALL:
			    time_start_rec_br_method_call=stat_elem.time;
			    if(time_yes_req_right_place!=0)
				interval=time_start_rec_br_method_call-time_yes_req_right_place;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TUSRBMC",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUSRBMC",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[7].addElement(new Long(interval));
			    /* TUSRBMC - time until start record+broadcast method call*/
			    break;
			case org.omg.FT.Utils.STOP_REC_BR_METHOD_CALL:
			    time_stop_rec_br_method_call=stat_elem.time;
			    if(time_start_rec_br_method_call!=0)
				interval=time_stop_rec_br_method_call-time_start_rec_br_method_call;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TRBMC",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRBMC",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[8].addElement(new Long(interval));
			    /* TRBMC - time to record+broadcast method call*/
			    break;
			case org.omg.FT.Utils.EXIT_REPLY_ALREADY_THERE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3)
				    possible_paths[j]=false;
			    time_exit_reply_there=stat_elem.time;
			    if(time_stop_rec_br_method_call!=0)
				interval=time_exit_reply_there-time_stop_rec_br_method_call;
			    else
				{
				    interval=0;
				}
			    av_sl_str=get_slice("TUFSR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUFSR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[9].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=4)
				    possible_paths[j]=false;
			    time_exit_receive_request=stat_elem.time;
			    if(time_stop_rec_br_method_call!=0)
				interval=time_exit_receive_request-time_stop_rec_br_method_call;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TUERR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[10].addElement(new Long(interval));
			    break;

			case org.omg.FT.Utils.ENTER_SEND_REPLY:
			    time_enter_send_reply=stat_elem.time;
			    if(time_exit_receive_request!=0)
				interval=time_enter_send_reply-time_exit_receive_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("ASP",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;//For now
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("ASP",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[11].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.START_REC_BR_REPLY:
			    time_start_rec_br_reply=stat_elem.time;
			    if(time_enter_send_reply!=0)
				interval=time_start_rec_br_reply-time_enter_send_reply;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TESRBR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TESRBR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[12].addElement(new Long(interval));
			    /* TESRBR - time from enter send reply to start record+broadcast reply in warm passive*/
			    break;
			case org.omg.FT.Utils.STOP_REC_BR_REPLY:
			    time_stop_rec_br_reply=stat_elem.time;
			    if(time_start_rec_br_reply!=0)
				interval=time_stop_rec_br_reply-time_start_rec_br_reply;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TRBR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRBR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[13].addElement(new Long(interval));
			    /* TRBR - time to record+broadcast reply in warm passive*/
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REPLY:
			    time_exit_send_reply=stat_elem.time;
			    if(time_stop_rec_br_reply!=0){
				interval=time_exit_send_reply-time_stop_rec_br_reply;
				av_sl_str=get_slice("TRRESR",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
				av_str.sum+=interval;
				av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRRESR",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[14].addElement(new Long(interval));
				/* TRRESR - time from end record reply to exit send reply*/
			    }
			    else{
				if(time_enter_send_reply!=0)
				    interval=time_exit_send_reply-time_enter_send_reply;
				else{
				    interval=0;
				}
				av_sl_str=get_slice("TSR",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
				av_str.sum+=interval;
				av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSR",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[15].addElement(new Long(interval));
			    }
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REPLY:
			    time_exit_receive_reply=stat_elem.time;
			    if(possible_paths[3])
				if(time_exit_reply_there!=0)
				    interval=time_exit_receive_reply-time_exit_reply_there;
				else{
				    interval=0;
				}
			    else
				if(time_exit_send_reply!=0)
				    interval=time_exit_receive_reply-time_exit_send_reply;
				else{
				    interval=0;
				}
			    av_sl_str=get_slice("TERR",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[16].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_OTHER:
			    time_send_other=stat_elem.time;
			    if(possible_paths[4])
				if(time_exit_receive_request!=0)
				    interval=time_send_other-time_exit_receive_request;
				else
				    interval=time_send_other-time_exit_send_request;
			    else {
				interval=0;
			    }
			    av_sl_str=get_slice("TSO",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSO",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[17].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION:
			    time_send_exception=stat_elem.time;
			    if(possible_paths[4])
				if(time_exit_receive_request!=0)
				    interval=time_send_exception-time_exit_receive_request;
				else
				    interval=time_send_exception-time_exit_send_request;
			    else{
				interval=0;
			    }
			    av_sl_str=get_slice("TSE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_str.sum+=interval;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSE",NR_CHARS_OPER)+"|"+interval+"\n");
			    vectors[18].addElement(new Long(interval));
			    break;
			case org.omg.FT.Utils.SEND_OTHER_RM:
			    time_send_other_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION_RM:
			    time_send_exception_rm=stat_elem.time;
			    break;
			case org.omg.FT.Utils.RECEIVE_OTHER:
			    time_receive_other=stat_elem.time;
			    if(time_send_other_rm!=0){
				interval=time_receive_other-time_send_other_rm;
				if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRORM",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[19].addElement(new Long(interval));
			    }
			    else
				if(time_send_other!=0){
				    interval=time_receive_other-time_send_other;
				 	if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
				    vectors[20].addElement(new Long(interval));
				}
				else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_other-time_send_req_exp_exc;
					if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_str.sum=total_time_crt;
					av_w_str.sum=total_time_w;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					vectors[20].addElement(new Long(interval));
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_other-time_send_wr_prim_exc;
					   	if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
					    vectors[20].addElement(new Long(interval));
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_other-time_send_fwd_exc;
						if(!decr){
						    av_str.nr_of_calls--;
						    av_w_str.nr_of_calls--;
						    decr=true;
						}
						av_w_str.sum=total_time_w;
						av_str.sum=total_time_crt;
						table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRO",NR_CHARS_OPER)+"|"+interval+"\n");
						vectors[20].addElement(new Long(interval));
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_other-time_exit_send_request;
						    if(!decr){
							av_str.nr_of_calls--;
							av_w_str.nr_of_calls--;
							decr=true;
						    }
						    av_w_str.sum=total_time_w;
						    av_str.sum=total_time_crt;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						    vectors[21].addElement(new Long(interval));
						}
			    break;
			case org.omg.FT.Utils.RECEIVE_EXCEPTION:
			    time_receive_exception=stat_elem.time;
			    if(time_send_exception_rm!=0){
				interval=time_receive_exception-time_send_exception_rm;
					if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRERM",NR_CHARS_OPER)+"|"+interval+"\n");
				vectors[22].addElement(new Long(interval));
			    }
			    else
				if(time_send_exception!=0){
				    interval=time_receive_exception-time_send_exception;
				   	if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
				    vectors[21].addElement(new Long(interval));
				}
				else
				    if(time_send_req_exp_exc!=0){
					interval=time_receive_other-time_send_req_exp_exc;
					  	if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					vectors[21].addElement(new Long(interval));
				    }
				    else
					if(time_send_wr_prim_exc!=0){
					    interval=time_receive_other-time_send_wr_prim_exc;
						if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					    vectors[21].addElement(new Long(interval));
					}
					else
					    if(time_send_fwd_exc!=0){
						interval=time_receive_other-time_send_fwd_exc;
						if(!decr){
					    av_str.nr_of_calls--;
					    av_w_str.nr_of_calls--;
					    decr=true;
					}
					av_w_str.sum=total_time_w;
					av_str.sum=total_time_crt;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
					vectors[21].addElement(new Long(interval));
					    }
					    else
						if(time_exit_send_request!=0){
						    interval=time_receive_exception-time_exit_send_request;
						    if(!decr){
							av_str.nr_of_calls--;
							av_w_str.nr_of_calls--;
							decr=true;
						    }
						    av_w_str.sum=total_time_w;
						    av_str.sum=total_time_crt;
						    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRE",NR_CHARS_OPER)+"|"+interval+"\n");
						    vectors[21].addElement(new Long(interval));
						}
			    break;
			}

		    }
		    break;
		case org.omg.FT.ACTIVE.value:
		case org.omg.FT.ACTIVE_WITH_VOTING.value:
		    possible_paths=new boolean[6];
		    for(i=0;i<6;i++)
			possible_paths[i]=false;
		    replica=0;
		    for(i=0;i<size;i++){
			stat_elem=(ReqTraceStr)ret_cl_id_vector.elementAt(i);
			switch(stat_elem.moment_code){
			case org.omg.FT.Utils.ENTER_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_enter_send_request=stat_elem.time;
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				possible_paths[j]=true;
			    time_exit_send_request=stat_elem.time;
			    if(time_enter_send_request!=0)
				interval=time_exit_send_request-time_enter_send_request;
			    else
				interval=0;
			    av_sl_str=get_slice("SCL",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SCL",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.SEND_REQ_EXP_EXC_GW:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=0)
				    possible_paths[j]=false;
			    time_send_req_exp_exc_gw=stat_elem.time;
			    if(time_exit_send_request!=0)
				interval=time_send_req_exp_exc_gw-time_exit_send_request;
			    else
				interval=0;
			    //av_str.sum+=interval;
			    av_sl_str=get_slice("TTGE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTGE",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.ENTER_RECEIVE_REQUEST_G:
			    possible_paths[0]=false;
			    time_enter_receive_request_gw=stat_elem.time;
			    if(time_exit_send_request!=0)
				interval=time_enter_receive_request_gw-time_exit_send_request;
			    else
				interval=0;
			    //    av_str.sum+=interval;
			    av_sl_str=get_slice("TEGI",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			   av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TEGI",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.BEG_SYNCH_FAILOVER:
			    possible_paths[0]=false;
			    time_beg_synch_failover=stat_elem.time;
			    if(time_enter_receive_request_gw!=0)
				interval=time_beg_synch_failover-time_enter_receive_request_gw;
			    else
				interval=0;
			    //av_str.sum+=interval;
			    av_sl_str=get_slice("TUSSF",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TUSSF",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.END_SYNCH_FAILOVER:
			    possible_paths[0]=false;
			    time_end_synch_failover=stat_elem.time;
			    if(time_beg_synch_failover!=0)
				interval=time_end_synch_failover-time_beg_synch_failover;
			    else
				interval=0;
			    //av_str.sum+=interval;
			     av_sl_str=get_slice("TSF",avg_vector);
			       if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TSF",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.GW_SEND_TRANSIENT:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=1)
				    possible_paths[j]=false;
			    time_gw_send_transient=stat_elem.time;
			    if(time_enter_receive_request_gw!=0)
				interval=time_gw_send_transient-time_enter_receive_request_gw;
			    else
				interval=0;
			    //av_str.sum+=interval;
			    av_sl_str=get_slice("TAGNA",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
 av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TAGNA",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			case org.omg.FT.Utils.GW_SEND_REQUESTS:
			    time_gw_send_requests=stat_elem.time;
			    possible_paths[0]=false;
			    possible_paths[1]=false;
			    if(time_end_synch_failover!=0)
				interval=time_gw_send_requests-time_end_synch_failover;
			    else
				interval=0;
			    //av_str.sum+=interval;
			    av_sl_str=get_slice("SRASF",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    //av_str.sum_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("SRASF",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* Send requests after (possible) synchronization in failover case*/
			    break;
			    /* start of active replica(s) processing part - it can appear several times,
			       equal with number of replicas*/
			case org.omg.FT.Utils.SEND_REQ_EXP_EXC:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=2)
				    possible_paths[j]=false;
			    time_send_req_exp_exc=stat_elem.time;
			    if(time_gw_send_requests!=0)
				interval=time_send_req_exp_exc-time_gw_send_requests;
			    else
				interval=0;
			    ////av_str.sum+=interval;
			    av_sl_str=get_slice("TTARE",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTARE",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* TTARE - time to travel to active replica and find out that the request expired*/
			    break;
			case org.omg.FT.Utils.ENTER_RECEIVE_REQUEST:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3&&j!=4&&j!=5)
				    possible_paths[j]=false;
			    time_enter_receive_request=stat_elem.time;
			    if(time_gw_send_requests!=0)
				interval=time_enter_receive_request-time_gw_send_requests;
			    else
				interval=0;

			    av_sl_str=get_slice("TGSURERR_"+stat_elem.replica_nr,avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TGSURERR",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* TGSURERR - time from gateway send req. until enter receive_request at replica (or, travel time form gw to replica)*/
			    break;
			case org.omg.FT.Utils.EXIT_REPLY_ALREADY_THERE:
			    for(j=0;j<possible_paths.length;j++)
				if(j!=3)
				    possible_paths[j]=false;
			    time_exit_reply_there=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_exit_reply_there-time_enter_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("TERRERT",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    ////av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TERRERT",NR_CHARS_OPER)+"|"+interval+"\n");
			    break;
			    /* TERRERT - time from enter receive request until exit reply there*/
			case org.omg.FT.Utils.EXIT_RECEIVE_REQUEST:
			    if(time_exit_receive_request!=0) continue;
			    time_exit_receive_request=stat_elem.time;
			    if(time_enter_receive_request!=0)
				interval=time_exit_receive_request-time_enter_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("TURSPR_"+stat_elem.replica_nr,avg_vector);
			    if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;

			    //av_str.sum+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TURSPR",NR_CHARS_OPER)+"|"+interval+"\n");
			    /*TURSPR - time until replica starts processing request*/
			    break;
			case org.omg.FT.Utils.ENTER_SEND_REPLY:
			    time_enter_send_reply=stat_elem.time;
			    for(j=0;j<possible_paths.length;j++)
				if(j!=4)
				    possible_paths[j]=false;
			    if(time_exit_receive_request!=0)
				interval=time_enter_send_reply-time_exit_receive_request;
			    else
				interval=0;
			    av_sl_str=get_slice("ASPAR_"+stat_elem.replica_nr,avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("ASPAR",NR_CHARS_OPER)+"|"+interval+"\n");
				/*ASPAR - approximate server processing time by active replica*/

			    break;
			case org.omg.FT.Utils.START_REC_REPLY:
			    time_start_rec_reply=stat_elem.time;
			    if(time_enter_send_reply!=0)
				interval=time_start_rec_reply-time_enter_send_reply;
			    else
				interval=0;
			    av_sl_str=get_slice("TESRRR_"+stat_elem.replica_nr,avg_vector);
			    if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TESRRR",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* TESRRR - time from enter send reply to record reply*/
			    break;
			case org.omg.FT.Utils.STOP_REC_REPLY:
			    time_stop_rec_reply=stat_elem.time;
			    if(time_start_rec_reply!=0)
				interval=time_stop_rec_reply-time_start_rec_reply;
			    else
				interval=0;
			    av_sl_str=get_slice("TRRS_"+stat_elem.replica_nr,avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;

			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRRS",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* TRR time to record reply stateless in act. repl.*/
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REPLY:
			    if(time_exit_send_reply!=0) continue;
			    time_exit_send_reply=stat_elem.time;
			    if(time_stop_rec_reply!=0)
				interval=time_exit_send_reply-time_stop_rec_reply;
			    else
				interval=0;
			     av_sl_str=get_slice("TRRESR_"+stat_elem.replica_nr,avg_vector);
			       if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRRESR",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* TRRESR - time from stop rec. reply until exit send reply*/
			    break;
			case org.omg.FT.Utils.SEND_EXCEPTION:
			    time_send_exception=stat_elem.time;
			    if(possible_paths[5])
				if(time_exit_receive_request!=0)
				    interval=time_send_exception-time_exit_receive_request;
				else
				    interval=0;
			    else
				interval=0;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TARSE",NR_CHARS_OPER)+"|"+interval+"\n");
			    /*TARSE - time from throw req. exp. exception or exit rec. req. until send exception by active replica*/
			    break;
			    /* end replica(s) processing part*/

			case org.omg.FT.Utils.RECEIVE_FIRST_REPLY:
			    time_receive_first_reply=stat_elem.time;
			    if(possible_paths[2]){
				if(time_send_req_exp_exc!=0)
				    interval=time_receive_first_reply-time_send_req_exp_exc;
				else
				    interval=0;
				av_sl_str=get_slice("TTGAFANP",avg_vector);
				if(av_sl_str.max_value<interval)
				    av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
				av_sl_str.sum_sq+=interval*interval;

				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTGAFANP",NR_CHARS_OPER)+"|"+interval+"\n");
				/* TTGAFANP  travel time to gateway from active replica after finding answer with no processing*/
			    }
			    else
				if(possible_paths[5]){
				    if(time_send_exception!=0)
					interval=time_receive_first_reply-time_send_exception;
				    else
					interval=0;
				    av_sl_str=get_slice("TTGARE",avg_vector);
				    if(av_sl_str.max_value<interval)
					av_sl_str.max_value=interval;
				    av_sl_str.sum+=interval;
				    av_sl_str.nr_of_calls++;
				    av_sl_str.sum_sq+=interval*interval;

				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTGARE",NR_CHARS_OPER)+"|"+interval+"\n");
				}
				else{
				    if(possible_paths[3])
					if(time_exit_reply_there!=0)
					    interval=time_receive_first_reply-time_exit_reply_there;
					else
					    interval=0;
				    else
					if(possible_paths[4])
					    if(time_exit_send_reply!=0)
						interval=time_receive_first_reply-time_exit_send_reply;
					    else
						interval=0;
				    //av_str.sum+=interval;
				    av_sl_str=get_slice("TTGAR",avg_vector);
				    if(av_sl_str.max_value<interval)
					av_sl_str.max_value=interval;
				    av_sl_str.sum+=interval;
				    av_sl_str.nr_of_calls++;
				    av_sl_str.sum_sq+=interval*interval;

				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TTGAR",NR_CHARS_OPER)+"|"+interval+"\n");
				}
			    /* TTGAR  travel time to gateway from active replica after processing*/

			    /* TTGARE  travel time to gateway from active replica after processing but when an exception is generated*/
			    interval=time_receive_first_reply-time_gw_send_requests;
			    av_sl_str=get_slice("TSRRFR",avg_vector);
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			    av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    break;
			case org.omg.FT.Utils.EXIT_SEND_REPLY_GW:
			    time_exit_send_reply_gw=stat_elem.time;
			    if(time_receive_first_reply!=0)
				interval=time_exit_send_reply_gw-time_receive_first_reply;
			    else
				interval=0;
			    av_sl_str=get_slice("TRSRC",avg_vector);
			      if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRSRC",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* time from receive first_reply until reply starts return to client*/
			    break;
			case org.omg.FT.Utils.EXIT_RECEIVE_REPLY:
			    time_exit_receive_reply=stat_elem.time;
			    if(time_exit_send_reply_gw!=0)
				interval=time_exit_receive_reply-time_exit_send_reply_gw;
			    else
				interval=0;
			     av_sl_str=get_slice("NRGTT",avg_vector);
			       if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
			    av_sl_str.sum+=interval;
			    av_sl_str.nr_of_calls++;
			     av_sl_str.sum_sq+=interval*interval;
			    rtt_without_wait+=interval;
			    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("NRGTT",NR_CHARS_OPER)+"|"+interval+"\n");
			    /* NRGTT - normal reply from gateway travel time (from exit send_reply, until it gets to the client)*/
			    break;
			case org.omg.FT.Utils.RECEIVE_EXCEPTION:
			    time_receive_exception=stat_elem.time;
			    if(possible_paths[0]){
				if(time_send_req_exp_exc_gw!=0)
				    interval=time_receive_exception-time_send_req_exp_exc_gw;
				else
				    interval=0;
				av_sl_str=get_slice("TREE",avg_vector);
				  if(av_sl_str.max_value<interval)
				av_sl_str.max_value=interval;
				av_sl_str.sum+=interval;
				av_sl_str.nr_of_calls++;
 				av_sl_str.sum_sq+=interval*interval;

			    rtt_without_wait+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TREE",NR_CHARS_OPER)+"|"+interval+"\n");
				/* TREE - time to receive exception req. exp. from gateway alone*/
			    }
			    else
				if(possible_paths[1]){
				    if(time_gw_send_transient!=0)
					interval=time_receive_exception-time_gw_send_transient;
				    else
					interval=0;
				    //av_str.sum+=interval;

				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TRET",NR_CHARS_OPER)+"|"+interval+"\n");
				    /* TRET -time to receive transient exc. from gateway*/
				}
				else
				    if(possible_paths[2]||possible_paths[5]){
					if(time_receive_first_reply!=0)
					    interval=time_receive_exception-time_receive_first_reply;
					else
					    interval=0;
					//av_str.sum+=interval;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TREEA",NR_CHARS_OPER)+"|"+interval+"\n");
					/* TREEA time to receive req. exp. exc. from gateway, but because the request expired at the active replica arrival*/
				    }
			    break;
			case org.omg.FT.Utils.RECEIVE_OTHER:
			    time_receive_other=stat_elem.time;
			    if(possible_paths[0]){
				if(time_send_req_exp_exc_gw!=0)
				    interval=time_receive_other-time_send_req_exp_exc_gw;
				else
				    interval=0;
				rtt_without_wait+=interval;
				//av_str.sum+=interval;
				table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TROE",NR_CHARS_OPER)+"|"+interval+"\n");
			    }
			    else
				if(possible_paths[1]){
				    if(time_gw_send_transient!=0)
					interval=time_receive_other-time_gw_send_transient;
				    else
					interval=0;
				    rtt_without_wait+=interval;
				    table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TROT",NR_CHARS_OPER)+"|"+interval+"\n");
				}
				else
				    if(possible_paths[2]||possible_paths[5]){
					if(time_receive_first_reply!=0)
					    interval=time_receive_other-time_receive_first_reply;
					else
					    interval=0;
					rtt_without_wait+=interval;
					table.writeBytes("   "+"|"+spaces(NR_DIGITS_NR)+"|"+spaces(client_id.length())+"|"+pad_str("TROEA",NR_CHARS_OPER)+"|"+interval+"\n");
				    }
			    /* the same meanings as for RECEIVE_EXCEPTION, but in case other is received*/
			    break;
			}

		    }
		    break;
		}
		av_str.sum_without_wait+=rtt_without_wait;
		av_str.sum_sq_without_wait+=rtt_without_wait*rtt_without_wait;
		if(av_str.max_value<rtt_without_wait)
		    av_str.max_value=rtt_without_wait;
		if(put) {
		    oper_av_t.put(oper_name,avg_vector);
		}
	    }
	    table.close();

	}catch(java.lang.Exception e){e.printStackTrace();}
    }

    String names_cp[]={"SCL","TTS","SUSFE","SUSWPE","SUSREE","TUDRRP","TUSRMC","TRMC","TUFSR","TUERR","ASP","TESRRR","TRR","TRRESR","TSR","TERR","TSO","TSE","TRORM","TRO","TRE","TRERM","TW"}, names_wp[]={"TW","SCL","TTS","SUSFE","SUSWPE","SUSREE","TUDRRP","TUSRBMC","TRBMC","TUFSR","TUERR","ASP","TESRBR","TRBR","TRRESR","TSR","TERR","TSO","TSE","TRORM","TRO","TRE","TRERM"};


     private void generate_files(String directory, int repl_style){
	java.util.Set entries;
	java.util.Iterator e_iter;
	java.util.Map.Entry elem;
	java.util.Vector[] vectors;
	String names[];
	String op_n;
	int i,j,l;
	try{
	    if(repl_style==org.omg.FT.COLD_PASSIVE.value)
		names=names_cp;
	    else
		names=names_wp;
	    entries=times.entrySet();
	    e_iter=entries.iterator();
	    java.io.RandomAccessFile to_write;
	    while(e_iter.hasNext()){
		elem=(java.util.Map.Entry)e_iter.next();
		op_n=(String)elem.getKey();
		vectors=(java.util.Vector[])elem.getValue();
		for(i=0;i<vectors.length;i++){
		    l=vectors[i].size();
		    if(l>0){
			to_write=new java.io.RandomAccessFile(directory+"/"+op_n+"_"+names[i],"rw");
			for(j=0;j<l;j++)
			    to_write.writeBytes(vectors[i].elementAt(j)+"\n");

		    }
		}
	    }
	}catch(java.lang.Exception e){}

    }

/** Method used to write a table with the times spent by the Replication Manager in failover procedures; the time is sliced in time spent from receiving the
* failure notification until recovery of a backup starts, time spent in setting up a backup to primary; in case a new replica has to be created, the time spent in
* this operation is also computed based on the traces
*/

    public void compute_repl_m_times(String experim_name){
	/* create a text file in the form:
	   operation | replication style | nr. replicas at start | nr. replicas at end | time(ms)

	   cr_obj    |     cold passive  |        5              |            6        |   10

	      ...............................................
	*/
	try{
	    java.io.RandomAccessFile table=new java.io.RandomAccessFile(experim_name,"rw");
	    java.util.Set entrySet=repl_m_trace_t.entrySet();
	    java.util.Iterator iter=entrySet.iterator();
	    java.util.Map.Entry entry;
	    java.util.Vector group_id_vector;
	    int size,i;
	    ReplMTrStr stat_elem;
	    long when_rec_f_not=0,when_begin_rec=0,when_begin_cr_obj=0,when_end_cr_obj=0,when_end_rec=0,when_begin_get_state=0,when_begin_set_state=0,interval,gr_id;
	    int nr_repl_r_f_n=0,nr_repl_b_rec=0,nr_repl_b_cr_o=0,nr_repl_e_cr_o,nr_repl_e_rec;
	    FailOverT fot_crt;
	    java.util.Hashtable fot_t=new java.util.Hashtable();
	    int tot_time_oc=0,tot_time_noc=0;
	    boolean cr_obj=false;
	    table.writeBytes("Repl. Style|Nr. Repl. at Start|Nr. Repl. at End|   Operation   |     Time\n");
	    table.writeBytes("-------------------------------------------------------------------------\n\n");
	    while(iter.hasNext()){
		table.writeBytes("\n");
		entry=(java.util.Map.Entry)iter.next();
		group_id_vector=(java.util.Vector)entry.getValue();
		size=group_id_vector.size();
		gr_id=((Long)entry.getKey()).longValue();
		fot_crt=(FailOverT)fot_t.get(new Long(gr_id));
		if(fot_crt==null){
		    fot_crt=new FailOverT();
		    fot_t.put(new Long(gr_id),fot_crt);
		}
		for(i=0;i<size;i++){
		    stat_elem=(ReplMTrStr)group_id_vector.elementAt(i);
		    switch(stat_elem.moment_code){
		    case org.omg.FT.Utils.REPL_MAN_REC_F_NOT:
			fot_crt.nr_f_o_nocr++;
			when_rec_f_not=stat_elem.time;
			tot_time_oc=fot_crt.sum_oc;
			tot_time_noc=fot_crt.sum_noc;
			nr_repl_r_f_n=stat_elem.crt_nr_replicas;
			break;
		    case org.omg.FT.Utils.REPL_MAN_BEGIN_REC:
			when_begin_rec=stat_elem.time;
			interval=when_begin_rec-when_rec_f_not;
			nr_repl_b_rec=stat_elem.crt_nr_replicas;
			fot_crt.sum_oc+=interval;
			fot_crt.sum_noc+=interval;

			table.writeBytes(org.omg.FT.Utils.repl_s_string(stat_elem.repl_style)+"   |   "+nr_repl_r_f_n+"               |    "+stat_elem.crt_nr_replicas+"   |         "+"op_before_s_rec|"+interval+"\n");
			break;
		    case org.omg.FT.Utils.REPL_MAN_BEGIN_CR_OBJ:
			fot_crt.nr_f_o_ocr++;
			fot_crt.nr_f_o_nocr--;
			cr_obj=true;
			when_begin_cr_obj=stat_elem.time;
			nr_repl_b_cr_o=stat_elem.crt_nr_replicas;
			break;
		    case org.omg.FT.Utils.REPL_MAN_END_CR_OBJ:
			interval=stat_elem.time-when_begin_cr_obj;
			//fot_crt.sum_oc+=interval;
			fot_crt.sum_noc=tot_time_noc;
			table.writeBytes(org.omg.FT.Utils.repl_s_string(stat_elem.repl_style)+"   |   "+nr_repl_b_cr_o+"               |    "+stat_elem.crt_nr_replicas+"   |         "+"op_cr_obj|"+interval+"\n");
			break;
		    case org.omg.FT.Utils.REPL_MAN_END_REC:
			interval=stat_elem.time-when_begin_rec;
			if(!cr_obj) {
			    fot_crt.sum_oc=tot_time_oc;
			    fot_crt.sum_noc+=interval;
			}
			else{
			    fot_crt.sum_oc+=interval;
			    fot_crt.sum_noc=tot_time_noc;
			}
			cr_obj=false;
			table.writeBytes(org.omg.FT.Utils.repl_s_string(stat_elem.repl_style)+"   |   "+nr_repl_b_rec+"               |    "+stat_elem.crt_nr_replicas+"   |         "+"op_rec|"+interval+"\n");
			break;
		    case org.omg.FT.Utils.REPL_MAN_BEG_GET_STATE:
			when_begin_get_state=stat_elem.time;
			break;
		    case org.omg.FT.Utils.REPL_MAN_END_GET_STATE:
			interval=stat_elem.time-when_begin_get_state;
			table.writeBytes(org.omg.FT.Utils.repl_s_string(stat_elem.repl_style)+"   |   "+nr_repl_b_rec+"               |    "+stat_elem.crt_nr_replicas+"   |         "+"get_state_act|"+interval+"\n");
			break;
		    case org.omg.FT.Utils.REPL_MAN_BEG_SET_STATE:
			when_begin_set_state=stat_elem.time;
			break;
		    case org.omg.FT.Utils.REPL_MAN_END_SET_STATE:
			interval=stat_elem.time-when_begin_set_state;
			table.writeBytes(org.omg.FT.Utils.repl_s_string(stat_elem.repl_style)+"   |   "+nr_repl_b_rec+"               |    "+stat_elem.crt_nr_replicas+"   |         "+"set_state_act|"+interval+"\n");
			break;
		    }
		}
	    }
	    entrySet=fot_t.entrySet();
	    iter=entrySet.iterator();

	    while(iter.hasNext()){
		entry=(java.util.Map.Entry)iter.next();
		fot_crt=(FailOverT)entry.getValue();
		if(fot_crt.nr_f_o_ocr!=0)
		    table.writeBytes("Group "+entry.getKey()+" Average failover time when create new object is "+(double)(((double)fot_crt.sum_oc)/((double)fot_crt.nr_f_o_ocr))+"\n");
		if(fot_crt.nr_f_o_nocr!=0)
		    table.writeBytes("Group "+entry.getKey()+" Average failover time when not create new object is "+(double)(((double)fot_crt.sum_noc)/((double)fot_crt.nr_f_o_nocr))+"\n");
	    }

	    table.close();
	}catch(java.lang.Exception e){e.printStackTrace();}
    }

/** Method used to compute times spent in get_state/get_update or set_state/set_update; the times are sliced in the time spent when logging the method call (for get_*),
* this time including the wait time of get_* for update methods to finish execution, the time spent in sending the request to the called CORBA object, to execute the
* get_* method call on the CORBA object, and to receive the reply back, and log it
*/

    public void compute_get_state_times(String experim_name){

	try{
	    java.io.RandomAccessFile table=new java.io.RandomAccessFile(experim_name,"rw");
	    java.util.Vector get_state_trace_v;
	    int size,i;
	    GetStateTrStr stat_elem;
	    java.util.Set entrySet;
	    java.util.Map.Entry entry;
	    java.util.Iterator iter;
	    long time_start_get_state_call=0,time_stop_get_state_call=0,time_send_get_state_req=0,time_rec_get_state_req=0,time_send_get_state_reply=0,time_rec_get_state_reply=0,time_start_state_rec=0,time_stop_state_rec=0,time_start_get_update=0,time_stop_get_update=0,interval,time_start_get_s_wait=0,time_start_get_u_wait=0,time_stop_get_s_wait=0,time_stop_get_u_wait=0,time_start_set_state=0;

	    table.writeBytes("State size|Operation|Time   \n");
	    table.writeBytes("--------------------------------\n");
	    entrySet=get_state_trace_t.entrySet();
	    iter=entrySet.iterator();

	    while(iter.hasNext()){
		entry=(java.util.Map.Entry)iter.next();
		get_state_trace_v=(java.util.Vector)entry.getValue();
		size=get_state_trace_v.size();
		sort_g_s_vector_t(get_state_trace_v);
		for(i=0;i<size;i++){
		    stat_elem=(GetStateTrStr)get_state_trace_v.elementAt(i);
		    switch(stat_elem.moment_code){
		    case org.omg.FT.Utils.START_SET_STATE:time_start_set_state=stat_elem.time;break;
		    case org.omg.FT.Utils.STOP_SET_STATE:
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"        set_state|"+(stat_elem.time-time_start_set_state)+"\n");
			break;
		    case org.omg.FT.Utils.START_GET_STATE_CALL:
			time_start_get_state_call=stat_elem.time;
			break;
		    case org.omg.FT.Utils.SEND_GET_STATE_REQ:
			time_send_get_state_req=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"        in send_req|"+(time_send_get_state_req-time_start_get_state_call)+"\n");
			break;
		    case org.omg.FT.Utils.REC_GET_STATE_REQ:
			time_rec_get_state_req=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"  travel to replica|"+(time_rec_get_state_req-time_send_get_state_req)+"\n");
			break;
		    case org.omg.FT.Utils.SEND_GET_STATE_REPLY:
			time_send_get_state_reply=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"  execute get_state|"+(time_send_get_state_reply-time_rec_get_state_req)+"\n");
			break;
		    case org.omg.FT.Utils.REC_GET_STATE_REPLY:
			time_rec_get_state_reply=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"travel from replica|"+(time_rec_get_state_reply-time_send_get_state_reply)+"\n");
			break;
		    case org.omg.FT.Utils.STOP_GET_STATE_CALL:
			time_stop_get_state_call=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"    return in lo_re|"+(time_stop_get_state_call-time_rec_get_state_reply)+"\n");
			break;
		    case org.omg.FT.Utils.START_STATE_REC:
			time_start_state_rec=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"      start_rec_g_s|"+(time_start_state_rec-time_stop_get_state_call)+"\n");
			break;
		    case org.omg.FT.Utils.STOP_STATE_REC:
			time_stop_state_rec=stat_elem.time;
			table.writeBytes(pad_nr(stat_elem.size,7)+"|"+"         record g_s|"+(time_stop_state_rec-time_start_state_rec)+"\n\n");
			time_start_get_state_call=0;
			time_send_get_state_req=0;
			time_rec_get_state_req=0;
			time_send_get_state_reply=0;
			time_rec_get_state_reply=0;
			time_stop_get_state_call=0;
			time_start_state_rec=0;
			time_stop_state_rec=0;
			break;
		    }
		}
	    }
	    table.close();
	}catch(java.lang.Exception e){}
    }



    public  void compute_avgs(String f_n,String to_f){
	java.io.RandomAccessFile req_t=null;
	try{
	    req_t=new java.io.RandomAccessFile(f_n,"rw");
	}catch(java.lang.Exception e){req_t=null;}
	if(req_t==null) return;

	String line,op_name;
	int ind_b;
	java.util.Hashtable op_avg=new java.util.Hashtable();
	AvRecStr crt_oper;
	try{
	    long len=req_t.length();
	    req_t.readLine();
	    req_t.readLine();
	    req_t.readLine();
	    req_t.readLine();
	    while(req_t.getFilePointer()<len){
		line=req_t.readLine();
		ind_b=line.lastIndexOf('|');
		if(ind_b!=-1)
		    op_name=line.substring(ind_b+1);
		else
		    op_name="";
		crt_oper=(AvRecStr)op_avg.get(op_name);
		if(crt_oper==null&&op_name.length()>0){
		    crt_oper=new AvRecStr();
		    op_avg.put(op_name,crt_oper);
		}
		req_t.readLine();
		if(crt_oper!=null)
		    crt_oper.nr_of_calls++;
		while(line.length()!=0&&req_t.getFilePointer()<len){
		    line=req_t.readLine();
		    ind_b=line.lastIndexOf('|');
		    if(ind_b!=-1&&crt_oper!=null)
			crt_oper.sum+=Integer.parseInt(line.substring(ind_b+1));
		}
		if(line.length()!=0) {req_t.close();break;}
	    }
	    req_t.close();
	}catch(java.lang.Exception e){e.printStackTrace();}
	printAvg(op_avg,null,to_f,false);
    }

    private  void printAvg(java.util.Hashtable avg_t,java.util.Hashtable avg_w_t,String file,boolean correct){
	java.io.RandomAccessFile avg_f=null;
	try{
	    avg_f=new java.io.RandomAccessFile(file,"rw");
	}catch(java.lang.Exception e){avg_f=null;}

	java.util.Set entries;
	java.util.Iterator e_iter;
	java.util.Map.Entry elem;
	String op_n;
	AvRecStr av_r;
	AvWaitRecStr av_w_r;
	int nr_of_calls;
	long sum_i,av_s;

	entries=avg_t.entrySet();
	e_iter=entries.iterator();
	while(e_iter.hasNext()){
	    elem=(java.util.Map.Entry)e_iter.next();
	    op_n=(String)elem.getKey();
	    av_r=(AvRecStr)elem.getValue();
	    av_s=(long)(((double)av_r.sum)/((double)av_r.nr_of_calls));
	    if(avg_f!=null)
		try{
		    avg_f.writeBytes(op_n+" has an average time of "+av_s+" for a number of "+av_r.nr_of_calls+" calls\n");
		}catch(java.io.IOException e){}
	    else
		System.out.println(op_n+" has an average time of "+av_s+" for a number of "+av_r.nr_of_calls+" calls");
	}
	if(avg_w_t==null) return;
	entries=avg_w_t.entrySet();
	e_iter=entries.iterator();
	while(e_iter.hasNext()){
	    elem=(java.util.Map.Entry)e_iter.next();
	    op_n=(String)elem.getKey();
	    av_w_r=(AvWaitRecStr)elem.getValue();
	    av_s=(long)(((double)av_w_r.sum)/((double)av_w_r.nr_of_calls));
	    if(avg_f!=null)
		try{
		    avg_f.writeBytes(op_n+" has an average wait time of "+av_s+" for a number of "+av_w_r.nr_of_calls+" calls\n");
		}catch(java.io.IOException e){}
	    else
		System.out.println(op_n+" has an average wait time of "+av_s+" for a number of "+av_w_r.nr_of_calls+" calls");
	}
    }

    private  void printAvg(java.util.Hashtable avg_t,String file,boolean correct){
	java.io.RandomAccessFile avg_f=null;
	try{
	    avg_f=new java.io.RandomAccessFile(file,"rw");
	}catch(java.lang.Exception e){avg_f=null;}

	java.util.Set entries;
	java.util.Iterator e_iter;
	java.util.Map.Entry elem;
	java.util.Vector avg_vector;
	String op_n;
	AvRecStr av_r;
	AvWaitRecStr av_w_r;
	AvSliceRecStr av_sl_str;
	int nr_of_calls,l,i;
	float av_s,av_w,av_sl,av_s_absolute;
	float st_dev_w;
	double st_dev_sl,st_dev_s;
	entries=avg_t.entrySet();
	e_iter=entries.iterator();
	while(e_iter.hasNext()){
	    elem=(java.util.Map.Entry)e_iter.next();
	    op_n=(String)elem.getKey();
	    avg_vector=(java.util.Vector)elem.getValue();
	    av_r=(AvRecStr)avg_vector.elementAt(0);
	    av_w_r=(AvWaitRecStr)avg_vector.elementAt(1);
	    av_s_absolute=(float)(((double)av_r.sum)/((double)av_r.nr_of_calls));
	    if(av_r.nr_of_calls>1){
		if(correct){
		    av_r.sum=av_r.sum_without_wait/*-av_r.max_value*/;
		    av_r.sum_sq=av_r.sum_sq_without_wait/*-av_r.max_value*av_r.max_value*/;
		}
	    }
	    System.out.println("Roundtrip time max value for "+op_n+" is "+av_r.max_value);
	    av_s=(float)(((double)(av_r.sum)/(double)av_r.nr_of_calls));
	    st_dev_s=((double)(av_r.sum_sq)/((double)av_r.nr_of_calls))-(double)(av_s*av_s);
	    if(st_dev_s>0)
		st_dev_s=Math.sqrt(st_dev_s);
	    else
		st_dev_s=-1;
	    if(avg_f!=null)
		    try{
			avg_f.writeBytes(op_n+" "+av_r.nr_of_calls+" "+av_s+" st. dev. "+st_dev_s+"\n");
		}catch(java.io.IOException e){}
	    else
		System.out.println(op_n+" has an average time of "+av_s+" for a number of "+av_r.nr_of_calls+" calls");

	    av_w=(float)(((double)av_w_r.sum)/((double)av_w_r.nr_of_calls));
	    if(avg_f!=null)
		try{
		    if(av_s!=0)
			avg_f.writeBytes(op_n+" "+av_w_r.nr_of_calls+" "+((float)(100.0*(av_w/av_s_absolute)))+"% "+av_w+"\n");
		}catch(java.io.IOException e){}
	    else
		System.out.println(op_n+" has an average wait time of "+av_w+" for a number of "+av_w_r.nr_of_calls+" calls");
	    l=avg_vector.size();
	    for(i=2;i<l;i++){
		av_sl_str=(AvSliceRecStr)avg_vector.elementAt(i);
		av_sl=(float)(((double)av_sl_str.sum)/((double)av_sl_str.nr_of_calls));
		st_dev_sl=((double)(av_sl_str.sum_sq)/((double)av_sl_str.nr_of_calls))-(double)(av_sl*av_sl);
		if(st_dev_sl>0)
		    st_dev_sl=Math.sqrt(st_dev_sl);
		else
		    st_dev_sl=-1;
		if(avg_f!=null)
		    try{
			if(av_s!=0)
			    avg_f.writeBytes(op_n+" "+pad_str(av_sl_str.oper,NR_CHARS_OPER)+" "+av_sl_str.nr_of_calls+" "+((float)(100.0*(av_sl/av_s)))+"% "+av_sl+" st.dev. "+st_dev_sl+"\n");
		    }catch(java.io.IOException e){}
		else
		    System.out.println(op_n+" spent with operation "+av_sl_str.oper+" "+av_sl+" for a number of "+av_r.nr_of_calls+" calls");
	    }
	}
    }


/** The main method of the Compiler class has to receive some command line arguments: the path where the trace files are placed, the fault tolerance domain
* name, the file where to write the times spent by the Replication Manager when performing failover (note that this file can be empty in case no failover
* procedure was performed), the replication style, the file where to write request time data, the file where to write the get_* data, the directory path where
* the files containing the collection of timing values that will be used later for computing averages; the last argument is used to specify whether to compute
* besides averages, standard deviations
*/

    public static void main(String args[]){
	SampleTraceCompiler compiler=new SampleTraceCompiler();

	if(args.length<8){
	    System.out.println("Too few args, there should be 9");
	    return;
	}


	String path=args[0];

	compiler.parse_repl_m_trace(path+"/"+"repl_man%%"+args[1]);
	compiler.compute_repl_m_times(args[2]);



	java.io.File file=new java.io.File(path);

	current_repl_style=Integer.parseInt(args[3]);

	java.io.File[] its_files=file.listFiles();
	int i;
	for(i=0;i<its_files.length;i++){
	    if(its_files[i].getName().indexOf("%%")!=-1&&its_files[i].getName().indexOf("repl_man%%")==-1)
		compiler.parse_req_trace(path+"/"+its_files[i].getName());
	    else
		if(its_files[i].getName().indexOf("--")!=-1&&args.length>3)
		    compiler.parse_get_state_trace(path+"/"+its_files[i].getName());
	}
	compiler.compute_req_times(args[4]);
	if(current_repl_style!=org.omg.FT.ACTIVE.value)
	   compiler.generate_files(args[6],current_repl_style);
	boolean correct;

	if(Integer.parseInt(args[7])==1) correct=true;
	else
	    correct=false;
	if(current_repl_style==org.omg.FT.ACTIVE.value)
	    compiler.printAvg(oper_av_t,args[6],correct);
	compiler.compute_get_state_times(args[5]);

    }
}
